unit RegEx;

interface

uses Windows, SysUtils, Registry, Forms, Classes;

//  - '   '
function HKEY_CLASSES_ROOT   : integer;
function HKEY_CURRENT_USER   : integer;
function HKEY_LOCAL_MACHINE  : integer;
function HKEY_USERS          : integer;
function HKEY_CURRENT_CONFIG : integer;

function regRootKey       : integer;
function regAuthorName    : string;
function regSectionName   : string;
function regAuthorSection : string; // = regSectionName\regAuthorName
function regCurrentPath   : string;

//       :
// _ = : \\_
// FullPath    = regRootKey: regSectionName\regAuthorName\Application.ExeName\

//      :
// regCurrentPath := regSectionName\regAuthorName\Application.ExeName\
//   
//  regCurrentPath = HKEY_LOCAL_MACHINE: Software\Application.ExeName\

// regSetAuthorName:
//   : AuthorName
//  : regCurrentPath
function regSetAuthorName(const AuthorName : string) : string;
// regSetSectionName:
//   : SectionName
//  : regCurrentPath
function regSetSectionName(const SectionName : string) : string;

// regSetRootKey:
//   : HKEY
//  : HKEY,  ,  0
function regSetRootKey(RootKey : integer) : integer;

function StrToBool(s : string): boolean;
function BoolToStr(b : boolean) : string;

//   regWrite*  TRUE   

function regReadString(const Ident,Default : string) : string;
function regWriteString(const Ident,Value : string) : boolean;

function regReadBool(const Ident : string; Default : boolean) : boolean;
function regWriteBool(const Ident : string; const Value : boolean) : boolean;

function regReadFloat(const Ident : string; Default : extended) : extended;
function regWriteFloat(const Ident : string; const Value : extended) : boolean;

function regReadOrdinal(const Ident : string; Default : variant) : variant;
function regWriteOrdinal(const Ident : string; const Value : variant) : boolean;

function regReadCurrency(const Ident : string; Default : currency) : currency;
function regWriteCurrency(const Ident : string; const Value : currency) : boolean;

function regReadInteger(const Ident : string; Default : longint) : longint;
function regWriteInteger(const Ident : string; const Value : longint) : boolean;

function regReadDateTime(const Ident : string; Default : TDateTime) : TDateTime;
function regWriteDateTime(const Ident : string; const Value : TDateTime) : boolean;

function regReadDate(const Ident : string; Default : TDateTime) : TDateTime;
function regWriteDate(const Ident : string; const Value : TDateTime) : boolean;

function regReadTime(const Ident : string; Default : TDateTime) : TDateTime;
function regWriteTime(const Ident : string; const Value : TDateTime) : boolean;

function regReadBinaryData(const Ident : string; var Data; SizeOfData : integer) : integer;
function regWriteBinaryData(const Ident : string; var Data; SizeOfData : integer) : boolean;

// regEraseSection:     
//   : SectionName
//  : TRUE   ,  FALSE
function regEraseSection(Section : string) : boolean;
// regEraseKey:  ,    ( )
//    : TRegistry.DeleteValue
//      
//   : ValueName
//  : TRUE   ,  FALSE
function regEraseKey(const Ident: string) : boolean;

function regReadSections(Sections : TStrings) : boolean;

implementation

var
  gRootKey          : integer;
  gAuthorName       : string;
  gSectionName      : string;
  gProgramName      : string;

procedure CheckLastChar(var s : string);
begin
  s := Trim(s);
  if (s <> '') and (Copy(s,Length(s),1) <> '\') then s := s + '\';
end;

function HKEY_CLASSES_ROOT; begin Result := Windows.HKEY_CLASSES_ROOT end;
function HKEY_CURRENT_USER; begin Result := Windows.HKEY_CURRENT_USER end;
function HKEY_LOCAL_MACHINE; begin Result := Windows.HKEY_LOCAL_MACHINE end;
function HKEY_USERS; begin Result := Windows.HKEY_USERS end;
function HKEY_CURRENT_CONFIG; begin Result := Windows.HKEY_CURRENT_CONFIG end;

function regRootKey; begin Result := gRootKey end;
function regAuthorName; begin Result := gAuthorName end;
function regSectionName; begin Result := gSectionName end;
function regAuthorSection; begin Result := gSectionName + gAuthorName end;
function regCurrentPath; begin Result := regAuthorSection + gProgramName end;

function regSetAuthorName;
begin
  gAuthorName := AuthorName;
  CheckLastChar(gAuthorName);
  Result := regCurrentPath;
end;

function regSetSectionName;
begin
  gSectionName := SectionName;
  CheckLastChar(gSectionName);
  Result := regCurrentPath;
end;

function regSetRootKey;
begin
  if (RootKey = Windows.HKEY_CLASSES_ROOT) or
     (RootKey = Windows.HKEY_CURRENT_USER) or
     (RootKey = Windows.HKEY_LOCAL_MACHINE) or
     (RootKey = Windows.HKEY_USERS) or
     (RootKey = Windows.HKEY_CURRENT_CONFIG)
     then Result := RootKey
  else Result := 0;
  if Result <> 0 then gRootKey := Result;
end;

function StrToBool;
begin
  s := AnsiUpperCase(Trim(s));
  Result := FALSE;
  if (s = '1') or (s = 'Y') or (s = '+') or (s = '') then Result := TRUE;
  if (s = '') or (s = 'YES') or (s = 'TRUE') then Result := TRUE;
end;

function BoolToStr;
begin
  if b then Result := 'Y' else Result := 'N';
end;

function regReadString;
begin
  with TRegistry.Create do
    try
      RootKey := gRootKey;
      if KeyExists(regCurrentPath) and OpenKey(regCurrentPath,FALSE) then
        try
          Result := ReadString(Ident);
        except
          Result := Default;
        end
      else Result := Default;
    finally
      CloseKey;
      Free;
    end;
end;

function regWriteString;
begin
  Result := TRUE;
  with TRegistry.Create do
    try
      RootKey := gRootKey;
      OpenKey(regCurrentPath,TRUE);
      WriteString(Ident,Value);
      CloseKey;
      Free;
    except
      Result := FALSE;
      CloseKey;
      Free;
    end;
end;

function regReadBool;
begin
  with TRegistry.Create do
    try
      RootKey := gRootKey;
      if KeyExists(regCurrentPath) and OpenKey(regCurrentPath,FALSE) then
        try
          Result := ReadBool(Ident);
        except
          Result := Default;
        end
      else Result := Default;
    finally
      CloseKey;
      Free;
    end;
end;

function regWriteBool;
begin
  Result := TRUE;
  with TRegistry.Create do
    try
      RootKey := gRootKey;
      OpenKey(regCurrentPath,TRUE);
      WriteBool(Ident,Value);
      CloseKey;
      Free;
    except
      Result := FALSE;
      CloseKey;
      Free;
    end;
end;

function regReadFloat(const Ident : string; Default : extended) : extended;
begin
  with TRegistry.Create do
    try
      RootKey := gRootKey;
      if KeyExists(regCurrentPath) and OpenKey(regCurrentPath,FALSE) then
        try
          Result := ReadFloat(Ident);
        except
          Result := Default;
        end
      else Result := Default;
    finally
      CloseKey;
      Free;
    end;
end;

function regWriteFloat;
begin
  Result := TRUE;
  with TRegistry.Create do
    try
      RootKey := gRootKey;
      OpenKey(regCurrentPath,TRUE);
      WriteFloat(Ident,Value);
      CloseKey;
      Free;
    except
      Result := FALSE;
      CloseKey;
      Free;
    end;
end;

function regReadOrdinal(const Ident : string; Default : variant) : variant;
begin
  with TRegistry.Create do
    try
      RootKey := gRootKey;
      if KeyExists(regCurrentPath) and OpenKey(regCurrentPath,FALSE) then
        try
          Result := ReadInteger(Ident);
        except
          Result := Default;
        end
      else Result := Default;
    finally
      CloseKey;
      Free;
    end;
end;

function regWriteOrdinal;
begin
  Result := TRUE;
  with TRegistry.Create do
    try
      RootKey := gRootKey;
      OpenKey(regCurrentPath,TRUE);
      WriteInteger(Ident,integer(Value));
      CloseKey;
      Free;
    except
      Result := FALSE;
      CloseKey;
      Free;
    end;
end;

function regReadCurrency(const Ident : string; Default : currency) : currency;
begin
  with TRegistry.Create do
    try
      RootKey := gRootKey;
      if KeyExists(regCurrentPath) and OpenKey(regCurrentPath,FALSE) then
        try
          Result := ReadCurrency(Ident);
        except
          Result := Default;
        end
      else Result := Default;
    finally
      CloseKey;
      Free;
    end;
end;

function regWriteCurrency;
begin
  Result := TRUE;
  with TRegistry.Create do
    try
      RootKey := gRootKey;
      OpenKey(regCurrentPath,TRUE);
      WriteCurrency(Ident,Value);
      CloseKey;
      Free;
    except
      Result := FALSE;
      CloseKey;
      Free;
    end;
end;

function regReadInteger;  begin Result := regReadOrdinal(Ident,Default) end;
function regWriteInteger; begin Result := regWriteOrdinal(Ident,Value) end;

function regReadDateTime;  begin Result := regReadFloat(Ident,Default) end;
function regWriteDateTime; begin Result := regWriteFloat(Ident,Value) end;

function regReadDate;  begin Result := regReadFloat(Ident,Default) end;
function regWriteDate; begin Result := regWriteFloat(Ident,Value) end;

function regReadTime;  begin Result := regReadFloat(Ident,Default) end;
function regWriteTime; begin Result := regWriteFloat(Ident,Value) end;

function regReadBinaryData(const Ident : string; var Data; SizeOfData : integer) : integer;
begin
  Result := 0;
  with TRegistry.Create do
    try
      RootKey := gRootKey;
      if KeyExists(regCurrentPath) and OpenKey(regCurrentPath,FALSE)
      then Result := ReadBinaryData(Ident,Data,SizeOfData);
    finally
      CloseKey;
      Free;
    end;
end;

function regWriteBinaryData;
begin
  Result := TRUE;
  with TRegistry.Create do
    try
      RootKey := gRootKey;
      OpenKey(regCurrentPath,TRUE);
      WriteBinaryData(Ident,Data,SizeOfData);
      CloseKey;
      Free;
    except
      Result := FALSE;
      CloseKey;
      Free;
    end;
end;

{$WARNINGS OFF}
function regEraseSection;
var
  S : TStringList;
  i : integer;
begin
  try
    S := TStringList.Create;
    CheckLastChar(Section);
    if Section = '' then begin
      Result := FALSE;
      Exit;
    end;
    with TRegistry.Create do
      try
        RootKey := gRootKey;
        Result := OpenKey(Section,FALSE);
        if not Result then Exit;
        if HasSubkeys then begin
          GetKeyNames(s);
          for i := 0 to s.Count - 1 do regEraseSection(Section + S[i]);
          end
        else begin
          GetValueNames(S);
          for i := 0 to S.Count - 1 do Result := Result and DeleteValue(S[i]);
        end;
        CloseKey;
        DeleteKey(Section);
      except
        Result := FALSE;
        CloseKey;
        Free;
      end;
  finally
    S.Free;
  end;
end;
{$WARNINGS ON}

function regEraseKey;
begin
  with TRegistry.Create do
    try
      RootKey := gRootKey;
      Result := OpenKey(regCurrentPath,FALSE) and DeleteValue(Ident);
      CloseKey;
      Free;
    except
      Result := FALSE;
      CloseKey;
      Free;
    end;
end;

function regReadSections;
begin
  Result := TRUE;
end;


initialization

  gRootKey  := HKEY_LOCAL_MACHINE;
  gSectionName := 'SOFTWARE\';
  gProgramName := ChangeFileExt(ExtractFileName(Application.ExeName),'\');
  regSetAuthorName('');

end.

